#!/bin/bash
# Copyright Atomicorp 2021
# All Rights reserved

VERSION=1.1.0

# Planned
if [ -f /var/ossec/etc/malware-scanner.conf ]; then
	source /var/ossec/etc/malware-scanner.conf
else
	CLAMAV_SERVER=daemon
fi

###########################
# Logging
###########################
LOG_FILE="/var/ossec/logs/active-responses.log"
#echo "`date` $0 $1 $2 $3 $4 $5" >> ${LOG_FILE}

###########################
#functions
###########################
function log_event () {
	MSG=$1
	echo "`date "+%b %d %H:%M:%S"` `hostname` atomicorp-api: ${MSG}" >> ${LOG_FILE}
}

function update_clamav_signatures() {
	log_event "updating malware rules" 
        if [[ $OSTYPE == "aix"* ]]; then
                /opt/freeware/bin/freshclam
        else
                if [ ! -d /var/lib/clamav ]; then
                        mkdir /var/lib/clamav
                fi
                /usr/bin/freshclam
        fi
        log_event "completed malware rule update" 


}

function init_clamav_conf() {
	if [[ $OSTYPE == "aix"* ]]; then
		cp -f /var/ossec/modules/clamav/template/clamd.conf.template /opt/freeware/etc/clamd.conf
		cp -f /var/ossec/modules/clamav/template/freshclam.conf.template /opt/freeware/etc/freshclam.conf
		if [ ! -d /var/log ]; then
			mkdir -p /var/log
		fi

		if [ ! -f /var/log/messages ]; then
			touch /var/log/messages
		fi
	else
		cp -f /var/ossec/modules/clamav/template/clamd.conf.template /etc/clamd.conf
		cp -f /var/ossec/modules/clamav/template/freshclam.conf.template /etc/freshclam.conf
		if [  ! -d /var/lib/clamav ]; then
			mkdir /var/lib/clamav
		fi
		
	fi
}

function update_clamav_conf(){
	PREFIX=$1
	# Clear freshclam configs
	sed -i "s/^PrivateMirror.*/d" $PREFIX/etc/freshclam.conf
	sed -i "s/^DatabaseCustomURL.*/d" $PREFIX/etc/freshclam.conf

	# Get manager
	HUB_IP=$(grep -o -P '(?<=<address>).*(?=</address>)' /var/ossec/etc/ossec.conf)
	
	# Definitions
	DB_FILES="ASL-advanced.ldb ASL-apt.yara ASL-badmacro.ndb ASL-blacklist.ldb ASL-fraud.ndb ASL.hdb ASL-h.ndb ASL-honeypot.hdb ASL-honeypot-hex.ndb ASL-ignores.ign ASL.ldb ASL-phish.ndb ASL-policy-bad-extensions.cdb ASL-policy-dangerous-extensions.cdb ASL-rogue.hdb ASL_securiteinfo_javascript.ndb ASL-spamlist.ldb ASL.yara"

	echo "PrivateMirror https://${HUB_IP}//channels/rules/malware/" >> /etc/freshclam.conf
	for rule_file in ${DB_FILES}; do
		echo "DatabaseCustomURL https://${HUB_IP}/channels/rules/malware/${rule_file}" >> /etc/freshclam.conf
	done
	
	
}

function install_clamav(){
        if [[ $OSTYPE == "aix"* ]]; then
		echo
	else
		if ! rpm -q clamd ; then
			yum -y install clamd
		fi
	fi
}

###########################
# Main
###########################
# Command line variables
ACTION=$1
shift
shift
ARGS="$@"


if [[ "$ARGS" == "update"* ]]; then
	update_clamav_signatures

elif [[ "$ARGS" == "init-local"* ]]; then

	log_event "malware scanner initializing local config"

	if [[ $OSTYPE == "aix"* ]]; then
		init_clamav_conf
		update_clamav_signatures
	else
		install_clamav
		init_clamav_conf
		update_clamav_signatures
		systemctl restart clamav-daemon
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed"
		fi
		
	fi
# Planned
elif [[ "$ARGS" == "init-remote"* ]]; then
	log_event "malware scanner initializing local config"
	if [[ $OSTYPE == "aix"* ]]; then
		init_clamav_conf
		update_clamav_signatures
	else
		init_clamav_conf
		update_clamav_signatures
		systemctl restart clamav-daemon
		RETVAL=$?
		if [ $RETVAL -eq 0 ]; then
			log_event "malware scanner initialization completed successfully"
		else
			log_event "malware scanner initialization failed"
		fi
		
		
	fi



else 
        log_event "running malware scan on ${ARGS}" 
	if [[ $OSTYPE == "aix"* ]]; then
		TARGET=$(echo $ARGS | awk -F\( '{print $1}')
		CLAM_PATH=/opt/freeware/bin 
		CLAM_CONF=/etc/clamd-remote.conf
		if [ ! -f /etc/clamd-remote.conf ]; then
			CLAMAV_SERVER=local
		fi
		
	else
		TARGET=${ARGS}
		CLAM_PATH=/usr/bin
		CLAM_CONF=/etc/clamd.conf
		
	fi

	# Scan Type
	if [[ $CLAMAV_SERVER == "daemon" ]]; then
		# remote scanner
		RESULT=$($CLAM_PATH/clamdscan -c $CLAM_CONF --stream -i -m  --no-summary ${ARGS})
	else
		# local
		RESULT=$($CLAM_PATH/clamscan -i --no-summary ${TARGET})
	fi



	IFS=$'\n'
	for line in $RESULT; do
		if [[ $OSTYPE == "aix"* ]]; then
			DATE=$(date "+%b %d %H:%M:%S")
			echo "${DATE} `hostname` clamd: $line" >> /var/log/messages
		else

        		logger -t clamd $line
		fi

	done
	unset IFS
        log_event "completed malware scan on ${ARGS}" 

fi

